package com.redhat.thermostat.tools.eclipse.plugin.wizards;

import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;

import com.redhat.thermostat.tools.eclipse.plugin.Messages;
import com.redhat.thermostat.tools.eclipse.plugin.ProjectFinder;
import com.redhat.thermostat.tools.eclipse.plugin.ProjectFinder.Criteria;
import com.redhat.thermostat.tools.eclipse.plugin.SelectionUtils;

public class PluginXmlProjectSelectionPage extends WizardPage {

    static final String PAGE_NAME = "selectProjectForPluginXml"; //$NON-NLS-1$

    private Text containerText;

    private ISelection selection;

    public PluginXmlProjectSelectionPage(ISelection selection) {
        super(PAGE_NAME);
        setTitle(Messages.PluginXmlProjectSelectionPage_title);
        setDescription(Messages.PluginXmlProjectSelectionPage_description);
        this.selection = selection;
    }

    public void createControl(Composite parent) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        container.setLayout(layout);
        layout.numColumns = 3;
        layout.verticalSpacing = 9;
        Label label = new Label(container, SWT.NONE);
        label.setText(Messages.PluginXmlProjectSelectionPage_projectNameLabel);

        containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        containerText.setLayoutData(gd);
        containerText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                dialogChanged();
            }
        });

        String[] proposals = getProjectNamesForPluginXmlFile().toArray(new String[0]);
        SimpleContentProposalProvider projectProposalProvider = new SimpleContentProposalProvider(proposals);
        TextContentAdapter contentAdapter = new TextContentAdapter();
        ContentProposalAdapter proposalAdapter = new ContentProposalAdapter(containerText, contentAdapter, projectProposalProvider, null, null);
        proposalAdapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);

        Button button = new Button(container, SWT.PUSH);
        button.setText(Messages.PluginXmlProjectSelectionPage_browse);
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                handleBrowse();
            }
        });
        initialize();
        dialogChanged();

        setControl(container);
    }

    private void initialize() {
        IProject project = SelectionUtils.getSelectedProject(selection);
        if (project != null) {
            containerText.setText(project.getName());
        }
    }

    private void handleBrowse() {
        ContainerSelectionDialog dialog = new ContainerSelectionDialog(
                getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
                Messages.PluginXmlProjectSelectionPage_selectFileContainer);
        if (dialog.open() == ContainerSelectionDialog.OK) {
            Object[] result = dialog.getResult();
            if (result.length == 1) {
                containerText.setText(((Path) result[0]).toString());
            }
        }
    }

    private void dialogChanged() {
        String containerName = getContainerName();

        if (containerName.length() == 0) {
            updateStatus(Messages.PluginXmlProjectSelectionPage_fileContainerMustBeSpecified);
            return;
        }

        IResource container = ResourcesPlugin.getWorkspace().getRoot()
                .findMember(new Path(containerName));

        if (container == null
                || (container.getType() & (IResource.PROJECT | IResource.FOLDER)) == 0) {
            updateStatus(Messages.PluginXmlProjectSelectionPage_fileContainerMustExist);
            return;
        }
        if (!container.isAccessible()) {
            updateStatus(Messages.PluginXmlProjectSelectionPage_projectMustBeWritable);
            return;
        }
        updateStatus(null);
    }

    private void updateStatus(String message) {
        setErrorMessage(message);
        setPageComplete(message == null);
    }

    public String getContainerName() {
        return containerText.getText();
    }

    private List<String> getProjectNamesForPluginXmlFile() {
        ProjectFinder finder = new ProjectFinder();
        return finder.findProjectNames(new Criteria() {
            @Override
            public boolean matches(IProject project) {
                return project.isOpen();
            }
        });
    }
}
